Type: Package
Package: logger
Title: A Lightweight, Modern and Flexible Logging Utility
Version: 0.4.3
Date: 2026-08-23
Authors@R: c(
    person("Gergely", "Daróczi", , "daroczig@rapporter.net", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-3149-8537")),
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut",
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Jonathan", "Carroll", , "rpkg@jcarroll.com.au", role = "ctb", 
           comment = c(ORCID = "0000-0002-1404-5264")),       
    person("Spare Cores", role = "fnd"),
    person("System1", role = "fnd")
  )
Description: Inspired by the the 'futile.logger' R package and 'logging'
    Python module, this utility provides a flexible and extensible way of
    formatting and delivering log messages with low overhead.
License: MIT + file LICENSE
URL: https://daroczig.github.io/logger/
BugReports: https://github.com/daroczig/logger/issues
Depends: R (>= 4.0.0)
Imports: utils
Suggests: botor, cli, covr, crayon, devtools, glue, jsonlite, knitr,
        mirai (>= 1.3.0), ntfy, pander, parallel, R.utils, rmarkdown,
        roxygen2, RPushbullet, rsyslog, shiny, slackr (>= 1.4.1),
        syslognet, telegram, testthat (>= 3.0.0), withr
Enhances: futile.logger, log4r, logging
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/testthat/parallel: TRUE
Encoding: UTF-8
Config/roxygen2/version: 8.1.0
NeedsCompilation: no
Packaged: 2026-08-24 12:00:28 UTC; daroczig
Author: Gergely Daróczi [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3149-8537>),
  Hadley Wickham [aut] (ORCID: <https://orcid.org/0000-0003-4757-117X>),
  Jonathan Carroll [ctb] (ORCID: <https://orcid.org/0000-0002-1404-5264>),
  Spare Cores [fnd],
  System1 [fnd]
Maintainer: Gergely Daróczi <daroczig@rapporter.net>
Repository: CRAN
Date/Publication: 2026-08-24 16:20:18 UTC
Built: R 4.6.1; ; 2026-08-24 17:46:43 UTC; unix
