Most docstrings have been generated by LLMs and tools with minimal or no post-editing.
In alphabetical order:
    Anthropic/Claude: https://www.anthropic.com/, https://claude.ai/
    DeepSeek:         https://deep-seek.ai/
    Ollama:           https://ollama.com/
                      with models:
                          codegeex4
                          deepcoder
                          deepseek-coder-v2
                          granite-code

Code comments have been written by contributor(s).
