% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{log_namespaces}
\alias{log_namespaces}
\title{Looks up logger namespaces}
\usage{
log_namespaces()
}
\value{
character vector of namespace names
}
\description{
Looks up logger namespaces
}
